/*
Copyright (c) 2008-2009, Johannes Schlüter
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
  * Redistributions of source code must retain the above copyright notice, this
    list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name of Johannes Schlüter nor the names of the contributors may
    be used to endorse or promote products derived from this software without
    specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

This product includes PHP software, freely available from <http://www.php.net/software/>
PHP includes the Zend Engine, freely available at <http://www.zend.com>.
*/

#include <pthread.h>

#include "ha_php.h"
#include "ha_php_undef.h"

#include "pse_sapi.h"
#include "php_worker.h"

#ifndef ZTS
#error We need PHP with ZTS! Compile PHP --with-maintainer-zts
// lateron we might use another worker architecture for a single connection
// and single-threaded mode to be used with libmysqld
#endif

ha_php::ha_php(handlerton *hton, TABLE_SHARE *table_arg):handler(hton, table_arg)
{
    printf("CREATE HA INSTANCE (%lx - %lx)\n", (long)this, (long)current_thd);
    
    pthread_mutex_init(&params.caller_mtx, 0);
    pthread_mutex_init(&params.worker_mtx, 0);

    pthread_cond_init(&params.call_cond, 0);
    pthread_cond_init(&params.resp_cond, 0);

    params.errorcode = 0;
    params.table = &table;
    params.action = NO_ACTION;

    params.data = 0;

    pthread_mutex_lock(&params.caller_mtx);
    pthread_create( &worker_thread, NULL, &php_worker::go, &params);

    printf("1 - worker created\n");
    wait_worker();
}

ha_php::~ha_php() {
    printf("DESTROY HA INSTANCE (%lx - %lx)\n", (long)this, (long)current_thd);
    call_worker(FINISH, false);
    pthread_mutex_unlock(&params.caller_mtx);
    pthread_join(worker_thread, 0);
}

int ha_php::wait_worker()
{
    printf("1 -    wait!\n");
    pthread_cond_wait(&params.resp_cond, &params.caller_mtx);
    
    return params.errorcode;
}

int ha_php::call_worker(enum worker_action action, action_data *data, bool wait)
{
    params.action = action;
    params.data = data;
    printf("1 -    signal!\n");
    pthread_mutex_lock(&params.worker_mtx);
    pthread_cond_signal(&params.call_cond);
    pthread_mutex_unlock(&params.worker_mtx);

    if (wait) {
        return wait_worker();
    }

    return 0;
};

static const char *ha_php_exts[] = {
    0
};

const char **ha_php::bas_ext() const
{
    // TODO: We might ask PHP for file name extensions this gives a sa[fn]e place for custom indexes etc.
    return ha_php_exts;
}

int ha_php::create(const char *name, TABLE *form, HA_CREATE_INFO *info)
{
    /*
    zend_bool ret;
    zval *retval;

    php_start_request();

    zend_first_try {
        zval *arg2;
        MAKE_STD_ZVAL(arg2);
        array_init(arg2);

        for (Field **field=form->field ; *field ; field++) {
            zval *item;
            char* field_name = estrdup((*field)->field_name);
            ALLOC_INIT_ZVAL(item);
            array_init_size(item, 4);
            add_assoc_long(item, "field_type", (long)(*field)->type());
            add_assoc_long(item, "field_length", (*field)->field_length);
            add_assoc_string(item, "field_name", field_name, 0);
            add_assoc_long(item, "flags", (*field)->flags);
            add_index_zval(arg2, (*field)->field_index, item);
        }

        php_call_method(0, 0, 0, "create_table", &retval, name, arg2);

        convert_to_boolean(retval);

    } zend_catch {
        // TODO Handle
    } zend_end_try();

    ret = Z_BVAL_P(retval);

    php_finish_request();

    if (ret) {
        //zval_ptr_dtor(&retval);
        return 0;
    } else {
        //zval_ptr_dtor(&retval);
        return -1;
    }
     */
    return 0;
}

int ha_php::delete_table(const char* name)
{
    // TODO: Actually we should forst lookup in EG(function_table) for the function
    //       and call only when possible, oh and then cache the function pointer
/*
    php_start_request();

    zend_first_try {
        php_call_method(0, 0, 0, "delete_table", 0, name);
    } zend_catch {
        return -1;
    } zend_end_try();

    php_finish_request();

    return 0;
 */
    return -1;
}

int ha_php::rename_table(const char *from, const char *to)
{
    // TODO: Actually we should forst lookup in EG(function_table) for the function
    //       and call only when possible, oh and then cache the function pointer
/*
    php_start_request();

    zend_first_try {
        php_call_        pthread_mutex_t signal_mutex     = PTHREAD_MUTEX_INITIALIZER;method(0, 0, 0, "delete_table", 0, from, to);
    } zend_catch {
        return -1;
    } zend_end_try();

    php_finish_request();

    return 0;
*/
    return -1;
}

int ha_php::open(const char*name, int mode, uint test_if_locked)
{
    //thr_lock_data_init(&share->lock, &lock, (void*) this);
    struct open_action_data data;
    data.name = name;

    return call_worker(OPEN, &data);
}

int ha_php::close()
{
    // TODO: Tell worker
    return 0;
}

THR_LOCK_DATA **ha_php::store_lock(THD *thd,
                                     THR_LOCK_DATA **to,
                                     enum thr_lock_type lock_type)
{
    /*
   if (lock_type != TL_IGNORE && lock.type == TL_UNLOCK)
     lock.type=lock_type;
   *to++= &lock;
   */
   return to;
}

int ha_php::info(uint flag)
{
    stats.records = 2;
 /*
    if (countable) {
        zval *retval;
        zend_first_try {
            php_call_method(&obj_zval, 0, &countable, "count", &retval);
            if (retval) {
                convert_to_long_ex(&retval);
                stats.records = Z_LVAL_P(retval);
                zval_ptr_dtor(&retval);
            }
         } zend_catch {
            printf("Bailed out line %d\n", __LINE__);
            return -1;
         } zend_end_try();
    }
*/
    return 0;
}

int ha_php::extra(enum ha_extra_function operation)
{
    return 0;
}

int ha_php::rnd_init(bool scan)
{
    return call_worker(RND_INIT);
}

int ha_php::rnd_next(uchar* buf)
{
    struct rnd_next_action_data data;
    data.buf = buf;
    return call_worker(RND_NEXT, &data);
}

void ha_php::position(const uchar *buf)
{/*
    printf("POSITION\n");
    if (seekable && iter->funcs->get_current_key) {
        zend_first_try {
            char *str_key;
            uint str_key_len;
            ulong int_key;

            // TODO: We should allow strings here
            // Problem is that we then need to allocate something (zval, own structure, ...) here
            // so we can later see what kind of index was be used but I have no idea where to
            // free this then (even if casted to string we'd still have to free it).
            // During close()? - oh that's late!
            if (iter->funcs->get_current_key(iter, &str_key, &str_key_len, &int_key) == HASH_KEY_IS_LONG) {
                my_store_ptr(ref, sizeof(ulong), (my_off_t)int_key);
            }
        } zend_catch {
            printf("Bailed out line %d\n", __LINE__);
        } zend_end_try();
    }
    return;
  */
}

int ha_php::rnd_pos(uchar* buf, uchar* pos)
{
    return HA_ERR_WRONG_COMMAND;
    /*
    zval *zpos;
    printf("SEEK\n");
    if (!seekable || !iter->funcs->get_current_key) {
        return HA_ERR_WRONG_COMMAND;
    }

    zend_first_try {
        php_call_method(&obj_zval, 0, 0, "seek", 0, (uint)my_get_ptr(pos,ref_length));
        return current_to_mysql(buf);
    } zend_catch {
        printf("Bailed out line %d\n", __LINE__);
        return -1;
    } zend_end_try();
     */
}


zval *ha_php::get_row_zval(uchar *buf)
{
    /*
    zval *row;

    ALLOC_INIT_ZVAL(row);
    array_init(row);
    for (Field **field=table->field ; *field ; field++) {
        if ((*field)->is_null()) {
            add_assoc_null(row, (*field)->field_name);
        } else {
            char attribute_buffer[1024];
            String attribute(attribute_buffer, sizeof(attribute_buffer),
                   &my_charset_bin);
            (*field)->val_str(&attribute,&attribute);
            add_assoc_stringl(row, (*field)->field_name, attribute.c_ptr(), attribute.length(), 1);
        }
    }
    return row;
     */
    return 0;
}

int ha_php::write_row(uchar *buf)
{
    /*
    zval *row;

    if (!writable) {
        return HA_ERR_WRONG_COMMAND;
    }

    zend_first_try {
        row = get_row_zval(buf);
        php_call_method(&obj_zval, 0, &writable, "write", 0, row);
    } zend_catch {
        printf("Bailed out line %d\n", __LINE__);
        return -1;
    } zend_end_try();

    return 0;
     */
    return HA_ERR_WRONG_COMMAND;
}

int ha_php::update_row(const uchar *old_data, uchar *new_data)
{
    /*
    zval *row;

    if (!updatable) {
        return HA_ERR_WRONG_COMMAND;
    }

    zend_first_try {
        row = get_row_zval(new_data);
        php_call_method(&obj_zval, 0, &updatable, "update", 0, row);
    } zend_catch {
        printf("Bailed out line %d\n", __LINE__);
        return -1;
    } zend_end_try();

    return 0;;
     */
    return HA_ERR_WRONG_COMMAND;
}

int ha_php::delete_row(const uchar *buf)
{
    /*
    if (!deletable) {
        return HA_ERR_WRONG_COMMAND;
    }

    zend_first_try {
        php_call_method(&obj_zval, 0, &deletable, "delete", 0);
    } zend_catch {
        printf("Bailed out line %d\n", __LINE__);
        return -1;
    } zend_end_try();

    return 0;
     */
    return HA_ERR_WRONG_COMMAND;
}
