/*
Copyright (c) 2008-2009, Johannes Schlüter
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
  * Redistributions of source code must retain the above copyright notice, this
    list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name of Johannes Schlüter nor the names of the contributors may
    be used to endorse or promote products derived from this software without
    specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

This product includes PHP software, freely available from <http://www.php.net/software/>
PHP includes the Zend Engine, freely available at <http://www.zend.com>.
*/
#ifndef _HA_PHP_H
#define	_HA_PHP_H

#include "mysql_priv.h"
#include "mysql.h"
#include "ha_php_undef.h"

#include "php_worker.h"


class ha_php : public handler {
public:
    ha_php(handlerton *hton, TABLE_SHARE *table_arg);
    ~ha_php();

    const char **bas_ext() const;
    int create(const char *name, TABLE *form, HA_CREATE_INFO *info);
    int delete_table(const char* name);
    int rename_table(const char *from, const char *to);

    int open(const char*name, int mode, uint test_if_locked);
    int close();
    THR_LOCK_DATA **store_lock(THD *thd, THR_LOCK_DATA **to, enum thr_lock_type lock_type);
    int info(uint flag);
    int extra(enum ha_extra_function operation);
    int rnd_init(bool scan);
    int rnd_next(uchar *buf);

    const char *table_type() const { return "CSV"; }
    const char *index_type(uint inx) { return "NONE"; }

    int write_row(uchar *buf);
    int update_row(const uchar *old_data, uchar *new_data);
    int delete_row(const uchar *buf);

    void position(const uchar *buf);
    int rnd_pos(uchar* buf, uchar* pos);

    ulonglong table_flags() const {
        return (HA_NO_TRANSACTIONS | HA_REC_NOT_IN_SEQ | HA_NO_AUTO_INCREMENT |
                HA_BINLOG_ROW_CAPABLE | HA_BINLOG_STMT_CAPABLE);
    }
    ulong index_flags(uint idx, uint part, bool all_parts) const { return 0; }
private:
    pthread_t worker_thread;
    THR_LOCK_DATA lock;

    int wait_worker();
    int call_worker(enum worker_action, action_data *data = 0, bool wait = true);
    /*
    zend_function *writable;
    zend_function *updatable;
    zend_function *deletable;
    */
    zval *get_row_zval(uchar *buf);
    int col_to_mysql(uchar* buf, Field **field, zval **col);
    int current_to_mysql(uchar* buf);


    struct php_worker_params params;

    /*
    zend_function *countable;
    zend_function *seekable;

    zend_class_entry *ce;
    zval *obj_zval;
*/
};

#endif	/* _HA_PHP_H */

