/*
Copyright (c) 2008-2009 Johannes Schlüter
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
  * Redistributions of source code must retain the above copyright notice, this
    list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name of Johannes Schlüter nor the names of the contributors may
    be used to endorse or promote products derived from this software without
    specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

This product includes PHP software, freely available from <http://www.php.net/software/>
PHP includes the Zend Engine, freely available at <http://www.zend.com>.
*/

#include "ha_php.h"

#include "ha_php_undef.h"

#include "my_php_types.h"
#include "php_worker.h"

#include "php.h"
#include "php_main.h"
#include "zend.h"

#include "zend_interfaces.h"
#include "ext/spl/spl_array.h"
extern "C" {
#include "ext/date/php_date.h"
}

#include "pse_sapi.h"
#include "pse_php_interfaces.h"
#include "pse_call_php.h"

static zend_function *check_instance_and_get_func_ptr(zend_class_entry *ce, zend_class_entry *interface, const char *name, ulong name_size TSRMLS_DC)
{
    // TODO: MAke this a method to get rid of the TSRM in the signature
    zend_function *retval;

    if (!instanceof_function(ce, interface TSRMLS_CC)) {
        return 0;
    }

    if (zend_hash_find(&ce->function_table, const_cast<char*>(name), name_size, (void **) &retval) == SUCCESS) {
        return retval;
    }

    return 0;
}

php_worker::php_worker(struct php_worker_params *params TSRMLS_DC) {
#ifdef ZTS
    this->tsrm_ls = tsrm_ls;
#endif
    this->params = params;
}

php_worker::~php_worker() {
    if (obj_zval) {
        zval_ptr_dtor(&obj_zval);
    }
}

void* php_worker::go(void *params_v) {
    struct php_worker_params *params = (struct php_worker_params*)params_v;
    TSRMLS_FETCH();

    printf("2 - Starte PHP\n");
    pse_php_request_start(TSRMLS_C);

    zend_first_try {
        bool done = false;
        zend_file_handle file;
        file.filename = const_cast<char*>("/tmp/engine.php");
        file.opened_path = NULL;
        file.free_filename = 0;
        file.type = ZEND_HANDLE_FILENAME;
        // TODO: Handle errors in init script
        php_execute_script(&file TSRMLS_CC);

        php_worker *worker = new php_worker(params TSRMLS_CC);

        pthread_mutex_lock(&params->worker_mtx);

        do {
            printf("2 - lock call\n");
            pthread_mutex_lock(&params->caller_mtx);
            printf("2 - signal resp\n");
            pthread_cond_signal(&params->resp_cond);
            printf("2 - unlock call\n");
            pthread_mutex_unlock(&params->caller_mtx);

            printf("2 - Wait for signal\n");
            pthread_cond_wait(&params->call_cond, &params->worker_mtx);

            printf("2 -    action start (%d)!\n", params->action);
            params->errorcode = worker->inner_action_handler(params->action);
            printf("2 -    action done!\n");

            if (params->action == FINISH) {
                done=1;
            }
        } while (!done);
        pthread_mutex_unlock(&params->worker_mtx);
        delete worker;
    } zend_catch {
        printf("Bailed out line %d\n", __LINE__);
        // TODO: Handle this
    } zend_end_try();
    printf("2 -   PHP Shutdown\n");
    pse_php_request_shutdown();

    return 0;
}

int php_worker::inner_action_handler(enum worker_action action) {
    switch (action) {
        case NO_ACTION:
        default:
            printf("2 - Da is was komplett falsch %d\n", action);
            exit(1);
        case FINISH:
            return 0;
        case OPEN:
            return open(params->data);
        case RND_INIT:
            return rnd_init(params->data);
        case RND_NEXT:
            return rnd_next(params->data);
    }
}

int php_worker::open(action_data *data)
{
    pse_call_php_method(0, 0, 0, "open_table", &obj_zval TSRMLS_CC, ((open_action_data*)data)->name);

    if (!obj_zval || Z_TYPE_P(obj_zval) != IS_OBJECT || !instanceof_function(Z_OBJCE_P(obj_zval), zend_ce_traversable TSRMLS_CC)) {
        //zval_ptr_dtor(&obj_zval);
        obj_zval = 0;
        return 1;
    }

    ce = Z_OBJCE_P(obj_zval);

    writable  = check_instance_and_get_func_ptr(ce, ce_mysqlstorage_writable,  "write",  sizeof("write") TSRMLS_CC);
    updatable = check_instance_and_get_func_ptr(ce, ce_mysqlstorage_updatable, "update", sizeof("update") TSRMLS_CC);
    deletable = check_instance_and_get_func_ptr(ce, ce_mysqlstorage_deletable, "delete", sizeof("delete") TSRMLS_CC);

    countable = check_instance_and_get_func_ptr(ce, spl_ce_Countable,        "count", sizeof("count") TSRMLS_CC);
    seekable  = check_instance_and_get_func_ptr(ce, spl_ce_SeekableIterator, "seek",  sizeof("seek") TSRMLS_CC);

    return 0;
}

int php_worker::rnd_init(action_data *data /*bool scan*/)
{
    iter = ce->get_iterator(ce, obj_zval, 0 TSRMLS_CC);

    if (EG(exception)) {
        return -1;
    }

    iter_at_first = 1;
    if (iter->funcs->rewind) {
        iter->funcs->rewind(iter TSRMLS_CC);
        if (EG(exception)) {
            return -2;
        }
    }

    return 0;
}

// TODO: Probably we don't want to rely on the PHP type but on the field's type...
//       but it seems to work, so no change needed
int php_worker::col_to_mysql(uchar* buf, Field **field, zval **col)
{
    zend_class_entry *date_ce;
    
    switch (Z_TYPE_PP(col)) {
        case IS_NULL:
        default:
            (*field)->set_null((*field)->field_index);
            //*buf |= (1 << (*field)->field_index);
            break;
        case IS_LONG:
        case IS_BOOL:
            (*field)->store(Z_LVAL_PP(col), false);
            break;
        case IS_DOUBLE:
            (*field)->store(Z_DVAL_PP(col), false);
            break;
        case IS_OBJECT:
            date_ce = php_date_get_date_ce();
            if (Z_OBJCE_PP(col) == date_ce) {
                MYSQL_TIME time;
                php_date_obj *dateobj = (php_date_obj *) zend_object_store_get_object(*col TSRMLS_CC);
                time.year   = dateobj->time->y;
                time.month  = dateobj->time->m;
                time.day    = dateobj->time->d;
                time.hour   = dateobj->time->h;
                time.minute = dateobj->time->i;
                time.second = dateobj->time->s;
                time.time_type = MYSQL_TIMESTAMP_DATETIME;
                (*field)->store_time(&time, MYSQL_TIMESTAMP_DATETIME);
                break;
            }
            // else fall-through
        case IS_ARRAY:
        case IS_RESOURCE:
            convert_to_string(*col);
            // fall-through
        case IS_STRING:
            (*field)->store(Z_STRVAL_PP(col), Z_STRLEN_PP(col), &my_charset_latin1, CHECK_FIELD_WARN);
            break;
    }

    return 0;
}

int php_worker::current_to_mysql(uchar* buf)
{
    int i = 0;
    HashTable *ht;
    zval **data;
    TABLE *table = *params->table;

    iter->funcs->get_current_data(iter, &data TSRMLS_CC);
    if (Z_TYPE_PP(data) == IS_ARRAY) {
        //memset(buf, 0, table->s->null_bytes);
        memset(buf, 0, (*params->table)->s->null_bytes);
        ht = Z_ARRVAL_PP(data);

        for (Field **field=table->field ; *field ; field++) {
            zval **col;

            if (zend_hash_find(ht, (*field)->field_name, strlen((*field)->field_name) + 1, (void**) & col) == FAILURE) {
                *buf |= (1 << i);
            } else {
                col_to_mysql(buf, field, col);
            }
            i++;
        }
    } else {
        memset(buf, 256, table->s->null_bytes);
        *buf ^= 1;
        col_to_mysql(buf, table->field, data);
    }

    return 0;
}

int php_worker::rnd_next(action_data *data)
{
    if (!iter_at_first) {
        iter->funcs->move_forward(iter TSRMLS_CC);
    }
    iter_at_first = 0;

    if (iter->funcs->valid(iter TSRMLS_CC) != SUCCESS) {
        iter->funcs->dtor(iter TSRMLS_CC);
        return HA_ERR_END_OF_FILE;
    }

    return current_to_mysql(((rnd_next_action_data*)data)->buf);
}
