/*
Copyright (c) 2009, Johannes Schlüter
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
  * Redistributions of source code must retain the above copyright notice, this
    list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name of Johannes Schlüter nor the names of the contributors may
    be used to endorse or promote products derived from this software without
    specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

This product includes PHP software, freely available from <http://www.php.net/software/>
PHP includes the Zend Engine, freely available at <http://www.zend.com>.
*/

#ifndef _PHP_WORKER_H
#define	_PHP_WORKER_H

#include "zend_compile.h"

#include "my_php_types.h"
#include <pthread.h>

struct action_data{};

struct open_action_data : action_data {
    const char *name;
};

struct rnd_next_action_data : action_data {
    uchar *buf;
};

struct php_worker_params {
    pthread_mutex_t caller_mtx;
    pthread_cond_t call_cond;

    pthread_mutex_t worker_mtx;
    pthread_cond_t resp_cond;

    enum worker_action action;
    int errorcode;
    TABLE **table;

    action_data *data;
};

class php_worker {
public:
    static void* go(void*);

    int open(action_data *data);
    int rnd_init(action_data *data /*bool scan*/);
    int rnd_next(action_data *data);
private:
    php_worker(struct php_worker_params* TSRMLS_DC);
    virtual ~php_worker();

    int inner_action_handler(enum worker_action action);
    int col_to_mysql(uchar* buf, Field **field, zval **col);
    int current_to_mysql(uchar* buf);

    struct php_worker_params *params;
#ifdef ZTS
    void ***tsrm_ls;
#endif

    zend_class_entry *ce;
    zval *obj_zval;

    int iter_at_first;
    zend_object_iterator *iter;

    zend_function *writable;
    zend_function *updatable;
    zend_function *deletable;

    zend_function *countable;
    zend_function *seekable;
};

#endif	/* _PHP_WORKER_H */

