/*
Copyright (c) 2008-2009, Johannes Schlüter
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
  * Redistributions of source code must retain the above copyright notice, this
    list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name of Johannes Schlüter nor the names of the contributors may
    be used to endorse or promote products derived from this software without
    specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

This product includes PHP software, freely available from <http://www.php.net/software/>
PHP includes the Zend Engine, freely available at <http://www.zend.com>.
*/


/**
Define some wrappers over zend_call_method() which include mutex locking and
creation of parameter zvals
 */
#include "pthread.h"
#include "zend.h"
#include "zend_interfaces.h"

pthread_mutex_t MUTEX_php;

void pse_call_php_method(zval **obj, zend_class_entry *obj_ce, zend_function **fn_proxy, const char *fname, zval **retval TSRMLS_DC, zval* arg1=0, zval* arg2=0)
{
    int param_count = 0;
    if (arg2) {
        param_count = 2;
    } else if (arg1) {
        param_count = 1;
    }

    zend_call_method(obj, obj_ce, fn_proxy, const_cast<char*>(fname), strlen(fname), retval, param_count, arg1, arg2 TSRMLS_CC);

    switch (param_count) {
        case 2: zval_ptr_dtor(&arg2); /* fall-through */
        case 1: zval_ptr_dtor(&arg1); /* fall-through */
        default: break;
    }
}

void pse_call_php_method(zval **obj, zend_class_entry *obj_ce, zend_function **fn_proxy, const char *fname, zval **retval TSRMLS_DC, long arg1, zval* z_arg2=0)
{
    zval *z_arg1;

    MAKE_STD_ZVAL(z_arg1);
    ZVAL_LONG(z_arg1, arg1);

    pse_call_php_method(obj, obj_ce, fn_proxy, fname, retval TSRMLS_CC, z_arg1, z_arg2);
}

void pse_call_php_method(zval **obj, zend_class_entry *obj_ce, zend_function **fn_proxy, const char *fname, zval **retval TSRMLS_DC, const char *arg1, zval* z_arg2=0)
{
    zval *z_arg1;

    MAKE_STD_ZVAL(z_arg1);
    ZVAL_STRING(z_arg1, arg1, 1);

    pse_call_php_method(obj, obj_ce, fn_proxy, fname, retval TSRMLS_CC, z_arg1, z_arg2);
}

void pse_call_php_method(zval **obj, zend_class_entry *obj_ce, zend_function **fn_proxy, const char *fname, zval **retval TSRMLS_DC, const char*arg1, const char*arg2)
{
    zval *z_arg1, *z_arg2;

    MAKE_STD_ZVAL(z_arg1);
    ZVAL_STRING(z_arg1, arg1, 1);
    MAKE_STD_ZVAL(z_arg2);
    ZVAL_STRING(z_arg2, arg2, 1);

    pse_call_php_method(obj, obj_ce, fn_proxy, fname, retval TSRMLS_CC, z_arg1, z_arg2);
}
