/*
Copyright (c) 2009, Johannes Schlüter
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
  * Redistributions of source code must retain the above copyright notice, this
    list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name of Johannes Schlüter nor the names of the contributors may
    be used to endorse or promote products derived from this software without
    specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

This product includes PHP software, freely available from <http://www.php.net/software/>
PHP includes the Zend Engine, freely available at <http://www.zend.com>.
*/

#include "ha_php.h"
#include "ha_php_undef.h"
#include "php.h"
#include "php_main.h"
#include "pse_sapi.h"

static void pse_error_handler(int type, const char *error_filename, const uint error_lineno, const char *format, va_list args)
{
    MYSQL_ERROR::enum_warning_level level;
    char *message;

    switch (type) {
        case E_CORE_ERROR:
        case E_RECOVERABLE_ERROR:
        case E_PARSE:
        case E_COMPILE_ERROR:
        case E_ERROR:
        case E_USER_ERROR:
            level = MYSQL_ERROR::WARN_LEVEL_ERROR;
            break;
        case E_CORE_WARNING:
        case E_COMPILE_WARNING:
        case E_WARNING:
        case E_USER_WARNING:
        default:
            level = MYSQL_ERROR::WARN_LEVEL_WARN;
            break;
        case E_NOTICE:
        case E_STRICT:
        case E_DEPRECATED:
        case E_USER_NOTICE:
        case E_USER_DEPRECATED:
            level = MYSQL_ERROR::WARN_LEVEL_NOTE;
            break;
    }

    vspprintf(&message, 0, format, args);
    // TODO: Introduce PSE_G(thd) and use it here
    // push_warning_printf(current_thd, MYSQL_ERROR::WARN_LEVEL_WARN, 1, "%s in %s on line %d", message, error_filename, error_lineno);
    printf("%s in %s on line %d", message, error_filename, error_lineno);
    efree(message);
}

static handler *pse_create_handler(handlerton *hton,
                                    TABLE_SHARE *table,
                                    MEM_ROOT *mem_root)
{
  return new (mem_root) ha_php(hton, table);
}

static int pse_init_func(void *p)
{
  handlerton *pse_hton;

  pse_hton= (handlerton *)p;

  pse_hton->state= SHOW_OPTION_YES;
  pse_hton->db_type= DB_TYPE_UNKNOWN;
  pse_hton->create= pse_create_handler;
  pse_hton->flags= (HTON_CAN_RECREATE | HTON_SUPPORT_LOG_TABLES |
                     HTON_NO_PARTITION | HTON_ALTER_NOT_SUPPORTED);

  pse_php_module_startup();
  zend_error_cb = pse_error_handler;

  return 0;
}

static int pse_done_func(void *p)
{
    /*
    do_php_module_shutdown();*/
    return 0;
}

struct st_mysql_storage_engine pse_storage_engine=
{ MYSQL_HANDLERTON_INTERFACE_VERSION };

static char *pse_runtime_version = const_cast<char*>(PHP_VERSION);
static MYSQL_SYSVAR_STR(version, pse_runtime_version,
  PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_READONLY,
  "PHP Version.",
  NULL, NULL, NULL);

static struct st_mysql_sys_var* pse_system_variables[]= {
  MYSQL_SYSVAR(version)
};

mysql_declare_plugin(php)
{
  MYSQL_STORAGE_ENGINE_PLUGIN,
  &pse_storage_engine,
  "PHP",
  "Johannes Schlueter",
  "PHP-based storage engine",
  PLUGIN_LICENSE_BSD,
  pse_init_func, /* Plugin Init */
  pse_done_func, /* Plugin Deinit */
  0x0100 /* 1.0 */,
  NULL,                       /* status variables                */
  pse_system_variables,                       /* system variables                */
  NULL                        /* config options                  */
}
mysql_declare_plugin_end;
