/*
Copyright (c) 2008-2009, Johannes Schlüter
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
  * Redistributions of source code must retain the above copyright notice, this
    list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name of Johannes Schlüter nor the names of the contributors may
    be used to endorse or promote products derived from this software without
    specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

This product includes PHP software, freely available from <http://www.php.net/software/>
PHP includes the Zend Engine, freely available at <http://www.zend.com>.
*/

#include <main/php.h>
#include <main/SAPI.h>
#include <main/php_main.h>
#include <main/php_variables.h>
#include <main/php_ini.h>
#include <zend_ini.h>

#include "pse_php_interfaces.h"


static const char HARDCODED_INI[] =
    "html_errors=0\n"
    "register_argc_argv=0\n"
    "implicit_flush=1\n"
    "output_buffering=0\n"
    "max_execution_time=0\n"
    "max_input_time=-1\n\0";

static int pse_startup(sapi_module_struct *sapi_module)
{
    if (php_module_startup(sapi_module, NULL, 0)==FAILURE) {
        return FAILURE;
    }
    return SUCCESS;
}

static int pse_ub_write(const char *str, uint str_length TSRMLS_DC)
{
    /* TODO: What do we want to do with such stuff? */
    printf("%s", str);
    return str_length;
}

static void pse_flush(void *server_context)
{
    /* TODO: Check whether we have a problem if sapi_flush in SAPI.c returns failure */
}

static void pse_send_header(sapi_header_struct *sapi_header, void *server_context TSRMLS_DC)
{
}

static char* pse_read_cookies(TSRMLS_D)
{
    return NULL;
}

static void pse_register_variables(zval *track_vars_array TSRMLS_DC)
{
    php_import_environment_variables(track_vars_array TSRMLS_CC);
}

static void pse_log_message(char *message)
{
    /* TODO: Send to MySQL Log */
    fprintf (stderr, "%s\n", message);
}


sapi_module_struct pse_module = {
    "mysql_storage",               /* name */
    "MySQL Storage Engine",        /* pretty name */

    pse_startup,             /* startup */
    php_module_shutdown_wrapper,   /* shutdown */

    NULL,                          /* activate */
    NULL,                          /* deactivate */

    pse_ub_write,            /* unbuffered write */
    pse_flush,               /* flush */
    NULL,                          /* get uid */
    NULL,                          /* getenv */

    php_error,                     /* error handler */

    NULL,                          /* header handler */
    NULL,                          /* send headers handler */
    pse_send_header,         /* send header handler */

    NULL,                          /* read POST data */
    pse_read_cookies,        /* read Cookies */

    pse_register_variables,  /* register server variables */
    pse_log_message,         /* Log message */
    NULL,                          /* Get request time */
    NULL,                          /* Child terminate */

    STANDARD_SAPI_MODULE_PROPERTIES
};
/* }}} */


int pse_php_module_startup() {
    zend_llist global_vars;
#ifdef ZTS
    void ***tsrm_ls = NULL;
#endif

#ifdef HAVE_SIGNAL_H
#if defined(SIGPIPE) && defined(SIG_IGN)
    // TODO: DO *We* need this? Does it conflict with mysqld?

    signal(SIGPIPE, SIG_IGN); /* ignore SIGPIPE in standalone mode so
                                 that sockets created via fsockopen()
                                 don't kill PHP if the remote site
                                 closes it.  in apache|apxs mode apache
                                 does that for us!  thies@thieso.net
                                 20000419 */
#endif
#endif

#ifdef ZTS
    tsrm_startup(1, 1, 0, NULL);
    tsrm_ls = ts_resource(0);
//    *ptsrm_ls = tsrm_ls;
#endif

    sapi_startup(&pse_module);

    pse_module.ini_entries = malloc(sizeof(HARDCODED_INI));
    memcpy(pse_module.ini_entries, HARDCODED_INI, sizeof(HARDCODED_INI));

    pse_module.phpinfo_as_text = 1;

    if (pse_module.startup(&pse_module)==FAILURE) {
        return FAILURE;
    }

    zend_llist_init(&global_vars, sizeof(char *), NULL, 0);

    /* Set some Embedded PHP defaults */
    SG(options) |= SAPI_OPTION_NO_CHDIR;
    SG(request_info).argc=0;
    SG(request_info).argv=NULL;

    pse_register_interfaces(TSRMLS_C);

    return SUCCESS;
}

int pse_php_request_start(TSRMLS_D)
{
    if (php_request_startup(TSRMLS_C)==FAILURE) {
        php_module_shutdown(TSRMLS_C);
        return FAILURE;
    }

    SG(headers_sent) = 1;
    SG(request_info).no_headers = 1;
    php_register_variable("PHP_SELF", "-", NULL TSRMLS_CC);

    return SUCCESS;
}

void pse_php_request_shutdown()
{
    php_request_shutdown((void *) 0);
}

void pse_php_module_shutdown(TSRMLS_D)
{
    php_module_shutdown(TSRMLS_C);
    sapi_shutdown();
#ifdef ZTS
    tsrm_shutdown();
#endif
    if (pse_module.ini_entries) {
        free(pse_module.ini_entries);
        pse_module.ini_entries = NULL;
    }
}
